###############################################################################
#                                                                             #
# Copyright (c) 2019 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import torch

class Algorithm:

    def __init__(self):
        self.offset = 1

    def setup(self, parameters):
        self.offset = numpy.asscalar(parameters['offset'])
        return torch.cuda.is_available()

    def process(self, inputs, data_loaders, outputs):
        client_id = inputs['client_id'].data.value

        x = torch.FloatTensor(1, 1).fill_(numpy.asscalar(client_id)).cuda()
        y = torch.FloatTensor(1, 1).fill_(self.offset).cuda()
        z = x + y
        score = z.cpu().numpy().astype(numpy.int32)[0][0]

        scores = [
            {
                'template_identity': client_id,
                'score': score if client_id % 2 else -score
            },
            {
                'template_identity': 0,
                'score': -score if client_id % 2 else score
            }
        ]

        outputs['scores'].write({
          'client_identity': client_id,
          'scores': scores
          })

        return True
